import { Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { isString, isUndefined } from 'lodash';
import React, { useImperativeHandle, useState } from 'react';
import { useEvent, useEventCallback, useId, useIsomorphicLayoutEffect, useRefExtra, useResize } from '@react-devui/hooks';
import { getOffsetToRoot, toPx } from '@react-devui/utils';
import { cloneHTMLElement, registerComponentMate } from '../../utils';
import { useComponentConfig, useGlobalScroll, useLayout, usePrefixConfig } from '../root';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DAffix' });
function Affix(props, ref) {
    const { children, dTarget, dTop = 0, dZIndex } = useComponentConfig(COMPONENT_NAME, props);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { dPageScrollRef, dContentResizeRef } = useLayout();
    //#endregion
    //#region Ref
    const affixRef = useRefExtra(() => document.querySelector(`[data-affix="${uniqueId}"]`));
    const referenceRef = useRefExtra(() => document.querySelector(`[data-affix-reference="${uniqueId}"]`));
    const targetRef = useRefExtra(isUndefined(dTarget)
        ? () => dPageScrollRef.current
        : dTarget === false
            ? () => {
                var _a, _b;
                return (_b = (_a = affixRef.current) === null || _a === void 0 ? void 0 : _a.parentElement) !== null && _b !== void 0 ? _b : null;
            }
            : dTarget);
    //#endregion
    const uniqueId = useId();
    const [sticky, setSticky] = useState(false);
    const [positionStyle, setPositionStyle] = useState();
    const updatePosition = useEventCallback(() => {
        const offsetEl = sticky ? referenceRef.current : affixRef.current;
        if (targetRef.current && offsetEl) {
            const offsetRect = offsetEl.getBoundingClientRect();
            const targetTop = getOffsetToRoot(targetRef.current);
            const distance = isString(dTop) ? toPx(dTop, true) : dTop;
            setSticky(Math.ceil(targetRef.current.scrollTop) + distance >= getOffsetToRoot(offsetEl) - targetTop);
            setPositionStyle({
                width: offsetRect.width,
                height: offsetRect.height,
                top: (isUndefined(dTarget) ? targetTop : targetRef.current.getBoundingClientRect().top) + distance,
                left: offsetRect.left,
            });
        }
    });
    useIsomorphicLayoutEffect(() => {
        updatePosition();
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    const globalScroll = useGlobalScroll(updatePosition);
    useEvent(dPageScrollRef, 'scroll', updatePosition, { passive: true }, globalScroll);
    useEvent(targetRef, 'scroll', updatePosition, { passive: true }, globalScroll || isUndefined(dTarget));
    useResize(sticky ? referenceRef : affixRef, updatePosition);
    useResize(dContentResizeRef, updatePosition);
    useImperativeHandle(ref, () => ({
        sticky,
        updatePosition,
    }), [sticky, updatePosition]);
    return (_jsxs(_Fragment, { children: [sticky &&
                cloneHTMLElement(children, {
                    style: Object.assign(Object.assign({}, children.props.style), { visibility: 'hidden' }),
                    'aria-hidden': true,
                    'data-affix-reference': uniqueId,
                }), cloneHTMLElement(children, {
                style: sticky
                    ? Object.assign(Object.assign(Object.assign({}, children.props.style), positionStyle), { position: 'fixed', zIndex: dZIndex !== null && dZIndex !== void 0 ? dZIndex : `var(--${dPrefix}zindex-sticky)` }) : children.props.style,
                'data-affix': uniqueId,
            })] }));
}
export const DAffix = React.forwardRef(Affix);
