var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useEffect, useRef, useState } from 'react';
import { flushSync } from 'react-dom';
import { useAsync, useId } from '@react-devui/hooks';
import { CheckCircleFilled, CheckOutlined, CloseCircleFilled, CloseOutlined, DCustomIcon, ExclamationOutlined, WarningFilled, } from '@react-devui/icons';
import { checkNodeExist, getClassName, getUID } from '@react-devui/utils';
import { registerComponentMate } from '../../utils';
import { useComponentConfig, usePrefixConfig } from '../root';
function ease(k) {
    return 0.5 * (1 - Math.cos(Math.PI * k));
}
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DProgress' });
export function DProgress(props) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { dPercent = 0, dType = 'line', dStatus, dWave = false, dLineCap = 'round', dLinearGradient, dGapDegree = 0.55 * Math.PI, dLabel, dSize, dLineWidth = 8 } = _a, restProps = __rest(_a, ["dPercent", "dType", "dStatus", "dWave", "dLineCap", "dLinearGradient", "dGapDegree", "dLabel", "dSize", "dLineWidth"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const progressRef = useRef(null);
    //#endregion
    const dataRef = useRef({});
    const async = useAsync();
    const uniqueId = useId();
    const gradientId = `${dPrefix}progress-${uniqueId}`;
    const status = isUndefined(dStatus) && dPercent === 100 ? 'success' : dStatus;
    const theme = status === 'success' ? 'success' : status === 'warning' ? 'warning' : status === 'error' ? 'danger' : 'primary';
    const [percent, setPercent] = useState(dPercent);
    const [wavePercent, setWavePercent] = useState(0);
    const valuenowStroke = isUndefined(dLinearGradient) ? `var(--${dPrefix}color-${theme})` : `url(#${gradientId})`;
    let waveGradient = null;
    const [lineWidth, setLineWidth] = useState(0);
    useEffect(() => {
        if (progressRef.current) {
            const observer = new ResizeObserver(() => {
                flushSync(() => {
                    if (progressRef.current) {
                        const el = progressRef.current.querySelector(`[data-progress-svg="${uniqueId}"]`);
                        if (el) {
                            setLineWidth(el.clientWidth);
                        }
                    }
                });
            });
            observer.observe(progressRef.current);
            return () => {
                observer.disconnect();
            };
        }
    }, [uniqueId]);
    const drawLine = () => {
        const lineProps = {
            x1: dLineCap === 'butt' ? 0 : dLineWidth / 2,
            y1: '50%',
            y2: '50%',
            strokeWidth: dLineWidth,
            strokeLinecap: dLineCap,
        };
        const waveWidth = Math.max((lineWidth * (percent / 100)) / 20, 12);
        const waveX2 = lineProps.x1 + (dLineCap === 'butt' ? lineWidth : lineWidth - dLineWidth) * (wavePercent / 100);
        const waveX1 = Math.max(waveX2 - waveWidth, lineProps.x1);
        const id = getUID();
        waveGradient = (_jsxs("linearGradient", Object.assign({ id: id, x1: waveX1, y1: "50%", x2: waveX2, y2: "50%", gradientUnits: "userSpaceOnUse" }, { children: [_jsx("stop", { offset: "0%", stopColor: "#fff", stopOpacity: 0 }), _jsx("stop", { offset: "80%", stopColor: "#fff", stopOpacity: 0.12 }), _jsx("stop", { offset: "100%", stopColor: "#fff", stopOpacity: 0.16 })] })));
        return (_jsxs(_Fragment, { children: [_jsx("line", Object.assign({}, lineProps, { x2: dLineCap === 'butt' ? lineWidth : lineWidth - dLineWidth / 2, stroke: `var(--${dPrefix}background-color-indicator)` })), percent > 0 && (_jsx("line", Object.assign({}, lineProps, { x2: lineProps.x1 + (dLineCap === 'butt' ? lineWidth : lineWidth - dLineWidth) * (percent / 100), stroke: valuenowStroke }))), dWave && wavePercent > 0 && _jsx("line", Object.assign({}, lineProps, { x1: waveX1, x2: waveX2, stroke: `url(#${id})` }))] }));
    };
    const arcSize = dSize !== null && dSize !== void 0 ? dSize : 120;
    const getCoordinate = (angle) => {
        const c = arcSize / 2;
        const r = arcSize / 2 - dLineWidth;
        return [c + Math.sin(angle) * r, dLineWidth + (r - Math.cos(angle) * r)];
    };
    const drawArc = (arc) => {
        const r = arcSize / 2 - dLineWidth;
        const angle = arc.endAngle - arc.startAngle;
        return angle === 2 * Math.PI ? (_jsxs(_Fragment, { children: [drawArc(Object.assign(Object.assign({}, arc), { startAngle: 0, endAngle: 1 * Math.PI })), drawArc(Object.assign(Object.assign({}, arc), { startAngle: 1 * Math.PI, endAngle: 2 * Math.PI }))] })) : (_jsx("path", { d: `M ${getCoordinate(arc.startAngle).join()}A ${r} ${r} 0 ${angle > Math.PI ? 1 : 0} 1 ${getCoordinate(arc.endAngle).join()}`, fill: "none", stroke: arc.stroke, strokeWidth: dLineWidth, strokeLinecap: dLineCap }));
    };
    const drawArcWave = (arc) => {
        const [x1, y1] = getCoordinate(arc.startAngle);
        const [x2, y2] = getCoordinate(arc.endAngle);
        const id = getUID();
        waveGradient = (_jsxs("linearGradient", Object.assign({ id: id, x1: x1, y1: y1, x2: x2, y2: y2, gradientUnits: "userSpaceOnUse" }, { children: [_jsx("stop", { offset: "0%", stopColor: "#fff", stopOpacity: 0 }), _jsx("stop", { offset: "80%", stopColor: "#fff", stopOpacity: 0.12 }), _jsx("stop", { offset: "100%", stopColor: "#fff", stopOpacity: 0.16 })] })));
        return dWave && wavePercent > 0 && drawArc(Object.assign(Object.assign({}, arc), { stroke: `url(#${id})` }));
    };
    const drawCircle = () => {
        const waveEndAngle = 2 * Math.PI * (wavePercent / 100);
        const waveStartAngle = Math.max(waveEndAngle - (2 * Math.PI) / 20, 0);
        return (_jsxs(_Fragment, { children: [drawArc({ startAngle: 0, endAngle: 2 * Math.PI, stroke: `var(--${dPrefix}background-color-indicator)` }), percent > 0 && drawArc({ startAngle: 0, endAngle: 2 * Math.PI * (percent / 100), stroke: valuenowStroke }), drawArcWave({ startAngle: waveStartAngle, endAngle: waveEndAngle })] }));
    };
    const drawDashboard = () => {
        const endAngle = Math.PI - dGapDegree / 2;
        const startAngle = -endAngle;
        const waveEndAngle = startAngle + (endAngle - startAngle) * (wavePercent / 100);
        const waveStartAngle = Math.max(waveEndAngle - (2 * Math.PI) / 20, startAngle);
        return (_jsxs(_Fragment, { children: [drawArc({
                    startAngle,
                    endAngle,
                    stroke: `var(--${dPrefix}background-color-indicator)`,
                }), percent > 0 &&
                    drawArc({
                        startAngle,
                        endAngle: startAngle + (endAngle - startAngle) * (percent / 100),
                        stroke: valuenowStroke,
                    }), drawArcWave({ startAngle: waveStartAngle, endAngle: waveEndAngle })] }));
    };
    dataRef.current.waveLoopFn = (startTime) => {
        const time = window.performance.now();
        const elapsed = Math.min((time - startTime) / 1500, 1);
        const speed = ease(elapsed);
        if (wavePercent < percent) {
            setWavePercent(percent * speed);
            dataRef.current.clearWaveTid = async.requestAnimationFrame(() => {
                var _a, _b;
                (_b = (_a = dataRef.current).waveLoopFn) === null || _b === void 0 ? void 0 : _b.call(_a, startTime);
            });
        }
        else {
            setWavePercent(0);
            dataRef.current.clearWaveTid = async.setTimeout(() => {
                var _a, _b;
                (_b = (_a = dataRef.current).waveLoopFn) === null || _b === void 0 ? void 0 : _b.call(_a, window.performance.now());
            }, 2000);
        }
    };
    useEffect(() => {
        var _a, _b, _c, _d;
        (_b = (_a = dataRef.current).clearWaveTid) === null || _b === void 0 ? void 0 : _b.call(_a);
        if (dWave) {
            (_d = (_c = dataRef.current).waveLoopFn) === null || _d === void 0 ? void 0 : _d.call(_c, window.performance.now());
        }
    }, [dWave]);
    dataRef.current.transitionLoopFn = (startTime, startPercent) => {
        const time = window.performance.now();
        const elapsed = Math.min((time - startTime) / 300, 1);
        const speed = ease(elapsed);
        const newPercent = startPercent + (dPercent - startPercent) * speed;
        setPercent(newPercent);
        if (newPercent !== dPercent) {
            dataRef.current.clearTransitionTid = async.requestAnimationFrame(() => {
                var _a, _b;
                (_b = (_a = dataRef.current).transitionLoopFn) === null || _b === void 0 ? void 0 : _b.call(_a, startTime, startPercent);
            });
        }
    };
    useEffect(() => {
        var _a, _b, _c, _d;
        (_b = (_a = dataRef.current).clearTransitionTid) === null || _b === void 0 ? void 0 : _b.call(_a);
        (_d = (_c = dataRef.current).transitionLoopFn) === null || _d === void 0 ? void 0 : _d.call(_c, window.performance.now(), percent);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [dPercent]);
    const node = dType === 'line' ? drawLine() : dType === 'circle' ? drawCircle() : drawDashboard();
    return (_jsxs("div", Object.assign({}, restProps, { ref: progressRef, className: getClassName(restProps.className, `${dPrefix}progress`, `${dPrefix}progress--${dType}`), style: Object.assign(Object.assign({}, restProps.style), { width: dType === 'line' && isUndefined(dSize) ? '100%' : undefined }), role: "progressbar", "aria-valuenow": dPercent, "aria-valuemin": 0, "aria-valuemax": 100 }, { children: [_jsxs(DCustomIcon, Object.assign({ viewBox: dType === 'line' ? `0 0 ${lineWidth} ${dLineWidth}` : `0 0 ${arcSize} ${arcSize}`, "data-progress-svg": uniqueId, dSize: dType === 'line' ? ['100%', dLineWidth] : arcSize }, { children: [_jsxs("defs", { children: [waveGradient, !isUndefined(dLinearGradient) && React.cloneElement(dLinearGradient, { id: gradientId })] }), node] })), dLabel !== false && (_jsx("div", Object.assign({ className: getClassName(`${dPrefix}progress__label`, {
                    [`${dPrefix}progress__label--${status}`]: status,
                }) }, { children: checkNodeExist(dLabel)
                    ? dLabel
                    : status === 'success'
                        ? React.createElement(dType === 'line' ? CheckCircleFilled : CheckOutlined, { className: `${dPrefix}progress__label-icon` })
                        : status === 'warning'
                            ? React.createElement(dType === 'line' ? WarningFilled : ExclamationOutlined, { className: `${dPrefix}progress__label-icon` })
                            : status === 'error'
                                ? React.createElement(dType === 'line' ? CloseCircleFilled : CloseOutlined, { className: `${dPrefix}progress__label-icon` })
                                : `${dPercent}%` })))] })));
}
