var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { createElement as _createElement } from "react";
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { isNull, isUndefined } from 'lodash';
import React, { useCallback, useMemo, useRef, useState } from 'react';
import { useForceUpdate, useId } from '@react-devui/hooks';
import { FilterFilled, LoadingOutlined, SearchOutlined } from '@react-devui/icons';
import { findNested, getClassName, isSimpleArrayEqual } from '@react-devui/utils';
import { useDValue } from '../../hooks';
import { registerComponentMate, WINDOW_SPACE } from '../../utils';
import { DButton } from '../button';
import { DCheckbox } from '../checkbox';
import { DInput } from '../input';
import { DPopover } from '../popover';
import { DRadio } from '../radio';
import { useComponentConfig, usePrefixConfig, useTranslation } from '../root';
import { DVirtualScroll } from '../virtual-scroll';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DTable.Filter' });
function TableFilter(props, ref) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { dList, dSelected, dVisible, dLoading = false, dSearchable = false, dSearchValue, dMultiple = false, dCustomItem, dCustomSearch, dPopupClassName, onSelectedChange, onVisibleChange, onSearchValueChange, onScrollBottom, afterVisibleChange } = _a, restProps = __rest(_a, ["dList", "dSelected", "dVisible", "dLoading", "dSearchable", "dSearchValue", "dMultiple", "dCustomItem", "dCustomSearch", "dPopupClassName", "onSelectedChange", "onVisibleChange", "onSearchValueChange", "onScrollBottom", "afterVisibleChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const vsRef = useRef(null);
    //#endregion
    const dataRef = useRef({
        showSelected: dMultiple ? [] : null,
        prevSelected: dMultiple ? [] : null,
    });
    const [t] = useTranslation();
    const forceUpdate = useForceUpdate();
    const uniqueId = useId();
    const listId = `${dPrefix}table-filter-list-${uniqueId}`;
    const getItemId = (val) => `${dPrefix}table-filter-item-${val}-${uniqueId}`;
    const itemsMap = useMemo(() => {
        const items = new Map();
        const reduceArr = (arr) => {
            for (const item of arr) {
                items.set(item.value, item);
            }
        };
        reduceArr(dList);
        return items;
    }, [dList]);
    const [searchValue, changeSearchValue] = useDValue('', dSearchValue, onSearchValueChange);
    const canSelectItem = useCallback((item) => !item.disabled, []);
    const [_select, changeSelect] = useDValue(dMultiple ? [] : null, dSelected, (value) => {
        if (onSelectedChange) {
            if (dMultiple) {
                onSelectedChange(value, value.map((v) => itemsMap.get(v)));
            }
            else {
                onSelectedChange(value, isNull(value) ? null : itemsMap.get(value));
            }
        }
    }, dMultiple ? isSimpleArrayEqual : undefined);
    if (_select !== dataRef.current.prevSelected) {
        dataRef.current.showSelected = dataRef.current.prevSelected = _select;
    }
    const select = useMemo(() => (dMultiple ? new Set(dataRef.current.showSelected) : dataRef.current.showSelected), 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [dataRef.current.showSelected, dMultiple]);
    const hasSearch = searchValue.length > 0;
    const hasSelected = dMultiple ? select.size > 0 : !isNull(select);
    const _filterFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.filter;
    const filterFn = useCallback((item) => {
        const defaultTableFilterFn = (item) => {
            return item.label.includes(searchValue);
        };
        return _filterFn ? _filterFn(searchValue, item) : defaultTableFilterFn(item);
    }, [_filterFn, searchValue]);
    const sortFn = dCustomSearch === null || dCustomSearch === void 0 ? void 0 : dCustomSearch.sort;
    const searchList = useMemo(() => {
        if (!hasSearch) {
            return [];
        }
        const searchList = [];
        dList.forEach((item) => {
            if (filterFn(item)) {
                searchList.push(item);
            }
        });
        if (sortFn) {
            searchList.sort(sortFn);
        }
        return searchList;
    }, [dList, filterFn, hasSearch, sortFn]);
    const list = hasSearch ? searchList : dList;
    const [_noSearchFocusItem, setNoSearchFocusItem] = useState();
    const noSearchFocusItem = (() => {
        let focusItem;
        if (_noSearchFocusItem) {
            focusItem = itemsMap.get(_noSearchFocusItem.value);
            if (focusItem && canSelectItem(focusItem)) {
                return focusItem;
            }
        }
        if (hasSelected) {
            focusItem = findNested(dList, (item) => canSelectItem(item) && dMultiple ? select.has(item.value) : select === item.value);
        }
        if (isUndefined(focusItem)) {
            focusItem = findNested(dList, (item) => canSelectItem(item));
        }
        return focusItem;
    })();
    const [_searchFocusItem, setSearchFocusItem] = useState();
    const searchFocusItem = (() => {
        if (_searchFocusItem && findNested(searchList, (item) => canSelectItem(item) && item.value === _searchFocusItem.value)) {
            return _searchFocusItem;
        }
        if (hasSearch) {
            return findNested(searchList, (item) => canSelectItem(item));
        }
    })();
    const focusItem = hasSearch ? searchFocusItem : noSearchFocusItem;
    const changeFocusItem = (item) => {
        if (!isUndefined(item)) {
            hasSearch ? setSearchFocusItem(item) : setNoSearchFocusItem(item);
        }
    };
    const changeSelectByClick = (val) => {
        if (dMultiple) {
            const arr = [].concat(dataRef.current.showSelected);
            const index = arr.findIndex((v) => v === val);
            if (index !== -1) {
                arr.splice(index, 1);
            }
            else {
                arr.push(val);
            }
            dataRef.current.showSelected = arr;
        }
        else {
            dataRef.current.showSelected = val;
        }
        forceUpdate();
    };
    const vsPerformance = useMemo(() => ({
        dList: list,
        dItemSize: 32,
        dItemKey: (item) => item.value,
        dFocusable: canSelectItem,
    }), [canSelectItem, list]);
    return (_jsx(DPopover, Object.assign({ ref: ref, className: getClassName(dPopupClassName, `${dPrefix}table__filter-popup`), onClick: (e) => {
            e.stopPropagation();
        }, dVisible: dVisible, dTrigger: "click", dPlacement: "bottom-right", dArrow: false, dInWindow: WINDOW_SPACE, dContent: _jsxs(_Fragment, { children: [dSearchable && (_jsx(DInput, { className: `${dPrefix}table__filter-search`, dPrefix: _jsx(SearchOutlined, {}), dPlaceholder: t('Search'), onModelChange: (val) => {
                        changeSearchValue(val);
                    } })), dLoading && (_jsx("div", Object.assign({ className: getClassName(`${dPrefix}table__filter-loading`, {
                        [`${dPrefix}table__filter-loading--empty`]: list.length === 0,
                    }) }, { children: _jsx(LoadingOutlined, { dSize: list.length === 0 ? 18 : 24, dSpin: true }) }))), _jsx(DVirtualScroll, Object.assign({}, vsPerformance, { ref: vsRef, dFillNode: _jsx("li", {}), dItemRender: (item, index, { aria }) => {
                        const { label: itemLabel, value: itemValue, disabled: itemDisabled } = item;
                        const node = dCustomItem ? dCustomItem(item) : itemLabel;
                        let isSelected = false;
                        if (dMultiple) {
                            isSelected = select.has(itemValue);
                        }
                        else {
                            isSelected = select === itemValue;
                        }
                        return (_createElement("li", Object.assign({}, aria, { key: itemValue, id: getItemId(itemValue), className: getClassName(`${dPrefix}table__filter-option`, {
                                'is-disabled': itemDisabled,
                            }), title: itemLabel, role: "option", "aria-selected": isSelected, "aria-disabled": itemDisabled, onClick: () => {
                                if (!itemDisabled) {
                                    changeFocusItem(item);
                                    changeSelectByClick(itemValue);
                                }
                            } }),
                            React.createElement(dMultiple ? DCheckbox : DRadio, { dModel: isSelected, dDisabled: itemDisabled }),
                            _jsx("div", Object.assign({ className: `${dPrefix}table__filter-option-content` }, { children: node }))));
                    }, dFocusItem: focusItem, dSize: 200, dPadding: 4, onScrollEnd: onScrollBottom }, { children: ({ render, vsList }) => render(_jsx("ul", Object.assign({ id: listId, className: `${dPrefix}table__filter-list`, style: { pointerEvents: dLoading ? 'none' : undefined }, tabIndex: -1, role: "listbox", "aria-multiselectable": dMultiple, "aria-activedescendant": isUndefined(focusItem) ? undefined : getItemId(focusItem.value) }, { children: list.length === 0 ? (_jsx("li", Object.assign({ className: `${dPrefix}table__filter-empty` }, { children: _jsx("div", Object.assign({ className: `${dPrefix}table__filter-option-content` }, { children: t('No Data') })) }))) : (vsList) }))) }))] }), dFooter: _jsx(DPopover.Footer, { dActions: [
                _jsx(DButton, Object.assign({ onClick: () => {
                        dataRef.current.showSelected = dMultiple ? [] : null;
                        forceUpdate();
                    }, dType: "link" }, { children: t('Table', 'Reset') }), "$$reset"),
                'ok',
            ] }), onVisibleChange: (visible) => {
            onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(visible);
            if (!visible) {
                changeSelect(dataRef.current.showSelected);
            }
        }, afterVisibleChange: afterVisibleChange }, { children: _jsx("button", Object.assign({}, restProps, { className: getClassName(restProps.className, {
                'is-active': hasSelected,
            }) }, { children: _jsx(FilterFilled, { dSize: 12 }) })) })));
}
export const DTableFilter = React.forwardRef(TableFilter);
