export const [INDETERMINATE, CHECKED, UNCHECKED] = ['INDETERMINATE', 'CHECKED', 'UNCHECKED'];
export class AbstractTreeNode {
    constructor(origin) {
        Object.defineProperty(this, "origin", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: origin
        });
        Object.defineProperty(this, "_parent", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
    }
    get parent() {
        return this._parent;
    }
    get root() {
        let node = this;
        while (node.parent) {
            node = node.parent;
        }
        return node;
    }
    get isLeaf() {
        return !this.children;
    }
    get id() {
        return this._id;
    }
    get status() {
        return this._status;
    }
    get checked() {
        return this._status === CHECKED;
    }
    get unchecked() {
        return this._status === UNCHECKED;
    }
    get indeterminate() {
        return this._status === INDETERMINATE;
    }
    get disabled() {
        return this._disabled;
    }
    get enabled() {
        return !this._disabled;
    }
    updateStatus(checked, onlySelf = false) {
        if (this.children && !onlySelf) {
            this.children.forEach((child) => {
                child.updateStatus(checked, onlySelf);
            });
        }
        this._calculateStatus(checked);
    }
    _setParent(parent) {
        this._parent = parent;
    }
    _updateAncestors(checked, onlySelf) {
        if (this._parent && !onlySelf) {
            this._parent.updateStatus(checked, onlySelf);
        }
    }
}
